﻿#ifndef CHECKERBOARD_H
#define CHECKERBOARD_H

#include <vector>
#include <string>
#include <iostream>
#include <fstream>
#include "CheckerPiece.h"
#include "CheckerPosition.h"

using namespace std;

const int BOARD_SIZE = 8;

/// <summary>
/// Represents a standard checker board
/// </summary>
class CheckerBoard
{
public:
    std::string lastMove = "";
    CheckerPosition moveOldPos = CheckerPosition(-1, -1);
    CheckerPosition moveNewPos = CheckerPosition(-1, -1);
    CheckerPiece board[BOARD_SIZE][BOARD_SIZE];

    CheckerBoard();

    /// <summary>
    /// Load checkerboard from file
    /// </summary>
    /// <param name="filename">Path to the file</param>
    static CheckerBoard loadFromFile(string filename);

    /// <summary>
    /// Move the checker from one position to another
    /// </summary>
    /// <param name="from">Old position of the checker</param>
    /// <param name="to">New position of the checker</param>
    void moveChecker(CheckerPosition from, CheckerPosition to);

    /// <summary>
    /// Get the color of the checker at the given position
    /// </summary>
    /// <param name="pos">The position of the checker</param>
    /// <returns>Color of the checker (BLACK or WHITE)</returns>
    CheckerPiece getCheckerAt(CheckerPosition pos) const;

    /// <summary>
    /// Set checker at a given position
    /// </summary>
    /// <param name="pos">The position to place the checker to</param>
    /// <param name="checker">The type of the checker</param>
    void setCheckerAt(CheckerPosition pos, CheckerPiece checker);

    /// <summary>
    /// Print the checker board into the console
    /// </summary>
    void print() const;

    /// <summary>
    /// Generate all possible moves for the player
    /// </summary>
    /// <param name="isWhite">Whether the player plays white or not</param>
    /// <returns>A vector of all possible future board branches</returns>
    std::vector<CheckerBoard> generateLegalMoves(bool isWhite) const;

    /// <summary>
    /// Apply the future branch of the board
    /// </summary>
    /// <param name="move">A board copy representing a possible branch in the future</param>
    void applyMove(const CheckerBoard &move);

    /// <summary>
    /// Evaluate the board
    /// </summary>
    /// <returns>The score of the board</returns>
    int evaluate() const;

    /// <summary>
    /// Checks if the game is over
    /// </summary>
    /// <returns>True if the game is over</returns>
    bool isGameOver() const;

    /// <summary>
    /// Check if the move is valid
    /// </summary>
    /// <param name="oldPos">Previous position of the checker</param>
    /// <param name="newPos">New position of the checker</param>
    /// <param name="piece">Type of the checker</param>
    /// <returns>True if the move is valid</returns>
    bool isValidMove(CheckerPosition oldPos, CheckerPosition newPos, CheckerPiece piece) const;

    /// <summary>
    /// Whether the move is considered a valid capture
    /// </summary>
    /// <param name="pos">The old position of the checker</param>
    /// <param name="newpos">The new position of the checker</param>
    /// <param name="cpos">The position of the checker to be captured</param>
    /// <param name="piece">Current checker piece</param>
    /// <returns>True if the move is a valid capture</returns>
    bool isValidCapture(CheckerPosition pos, CheckerPosition newpos, CheckerPosition cpos, CheckerPiece piece) const;

private:
    /// <summary>
    /// Generate all possible moves for a piece
    /// </summary>
    /// <param name="position">The position of the piece</param>
    /// <param name="moves">A vector of possible alternative future boards</param>
    void generateMovesForPiece(CheckerPosition position, std::vector<CheckerBoard> &moves) const;

    /// <summary>
    /// Generate combination captures (>1 per move)
    /// </summary>
    /// <param name="position">The position of the checker</param>
    /// <param name="currentBoard">Current board</param>
    /// <param name="moves">Possible future moves</param>
    void generateAdditionalCaptures(CheckerPosition position, CheckerBoard &currentBoard, std::vector<CheckerBoard> &moves) const;
};

#endif